﻿<%@ Page Language="C#" MasterPageFile="~/SecurityExamples/SecurityMaster.master"
    AutoEventWireup="true" CodeFile="CSRFGet.aspx.cs" Inherits="CSRFGet" %>

<asp:Content ID="Content1" ContentPlaceHolderID="head" runat="Server">
    <title>CSRF - GET</title>
</asp:Content>
<asp:Content ID="Content2" ContentPlaceHolderID="ContentPlaceHolder1" runat="Server">
    <h2>
        CSRF - GET
    </h2>
    <div class="description">
        <p>
            A few years ago, Cross Site Request Forgery was not taken as a serious bug. It wasn&#39;t
            even considered to be a bug at all. But today, the web is about lots of money and
            many non-IT users manage important websites. That&#39;s why this kind of vulnerability
            is very popular these days. An important condition for a successful attack is that
            a user must click the attacker&#39;s link.</p>
        <p>
            A browser typically uses two ways of requesting web applications – sending data
            via URL parameters where a HTTP GET request is used, and sending data via forms
            where HTTP POST is used. The application typically performs some action – inserts
            a new user into a table, deletes a forum post, etc. Nothing strange? Yes, but …
            but there is one problem – the web application typically doesn&#39;t check if the
            requests are generated by the web application itself (= a user clicks a link or
            sends a filled form). Still seems okay? Let&#39;s continue. What if the attacker
            creates a link for some action and sends it to the user? The user clicks the link
            and the action is performed without the user even noticing. And this is called Cross
            Site Request Forgery.</p>
        <p>
            We already know that users have to click on the attacker&#39;s link or fill their
            form. Another condition is that the user must be logged on to the vulnerable website,
            but these days, almost every application provides &quot;keep me logged in&quot;
            functionality. ASP.NET makes it harder to perform a successful attack because of
            ViewState. If ViewState is turned on, you cannot send tampered POST requests to
            an ASP.NET application, because validation of ViewState fails.</p>
    </div>
    <h3>
        How it works
    </h3>
    <div class="works">
        <p>
            For this demo, you must be logged in as global administrator. So please log on <a
                href="localhost/MuniDemo/CMSPages/logon.aspx" target="_blank">here</a>.
        </p>
        <p>
            This example simulates the successful deletion of user through a malicious link
            sent to a user who is a global administrator.
        </p>
        <div>
            <a href="?userID=53">Click here for demo</a>
            <p>
                Result: <u>
                    <asp:Label ID="Label1" runat="server" Text=""></asp:Label></u>
            </p>
        </div>
    </div>
    <h3>
        How to protect against it
    </h3>
    <div class="protection">
        <p>
            Many developers think that an ASP.NET application is bulletproof against CSRF. But
            there are always a few catches:</p>
        <ul>
            <li>GET requests can still cause CSRF.</li>
            <li>ViewState can be generated outside the application if you are not using machine
                keys as keys for ViewState encoding.</li>
            <li>Even if you are using machine keys to encrypt your ViewState, you are not 100% safe.
                ASP.NET doesn&#39;t take form values from Request.Form, but from Request.Params.
                This is the reason why it&#39;s possible to perform something called a &quot;One
                click attack&quot;. It is a special case of CSRF. You simply send ViewState and
                values of form fields via GET. The trick is that you can use ViewState generated
                by ASP.NET after post, change the values of fields and validation still succeeds.</li>
        </ul>
    </div>
</asp:Content>
